unit _Mainform;

interface

uses
	RakBinaryStreamData, miscfunc,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Mask, ExtDlgs;

type
  TFrmMainForm = class(TForm)
    Panel1: TPanel;
    BtnNew: TButton;
    BtnOpen: TButton;
    BtnSaveAs: TButton;
    Label1: TLabel;
    EdtString: TEdit;
    Label2: TLabel;
    EdtInteger: TMaskEdit;
    Label3: TLabel;
    CckBoolean: TCheckBox;
    GroupBox1: TGroupBox;
    ImgPicture: TImage;
    BtnPicture: TButton;
    Odg: TOpenDialog;
    Sdg: TSaveDialog;
    OpdPicture: TOpenPictureDialog;
    CckStretch: TCheckBox;
    EdtFloat: TEdit;
    procedure BtnNewClick(Sender: TObject);
    procedure BtnOpenClick(Sender: TObject);
    procedure BtnSaveAsClick(Sender: TObject);
    procedure BtnPictureClick(Sender: TObject);
    procedure CckStretchClick(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  FrmMainForm: TFrmMainForm;

implementation

{$R *.DFM}

const
	APP_ID = 'SimpleRakBinaryStreamDataExample';

procedure TFrmMainForm.BtnNewClick(Sender: TObject);
begin
  EdtString.Text := '';
  EdtInteger.Text := '';
  EdtFloat.Text := '';
  CckBoolean.Checked := false;
  ImgPicture.Picture.Graphic := nil;
end;

procedure TFrmMainForm.BtnOpenClick(Sender: TObject);
var
	Data: TRakBinaryStreamData;
  Attr: TRakBinaryAttribute;
begin
	if Odg.Execute then
  begin
    Data := TRakBinaryStreamData.Create;
    try
      case Data.LoadFromFile(Odg.FileName) of
        blrOK:
        	begin
          	if Data.ApplicationName = APP_ID then
            begin
              EdtString.Text := Data.AttributeAsString('String-Wert', '');

              // Den Integerwert knnten wir auch mit Data.AttributeAsInteger abfragen.
              // Dann knnten wir aber nicht auf den Fall reagieren,
              // dass der Wert in der Datei gar nicht gespeichert war und das Editfeld leeren
              Attr := Data.AttributeByName('Integer-Wert');
              if Attr is TRakBinaryLongint then
                EdtInteger.Text := TRakBinaryLongint(Attr).AsWideString
              else
                EdtInteger.Text := '';

              Attr := Data.AttributeByName('Float-Wert');
              if Attr is TRakBinaryDouble then
                EdtFloat.Text := TRakBinaryDouble(Attr).AsWideString
              else
                EdtFloat.Text := '';

              CckBoolean.Checked := Data.AttributeAsBoolean('Boolean-Wert', false);

              Attr := Data.AttributeByName('Bild');
              if (Attr is TRakBinaryGraphic) and not TRakBinaryGraphic(Attr).IsNull then
              	ImgPicture.Picture.Graphic := TRakBinaryGraphic(Attr).Graphic
              else
              	ImgPicture.Picture.Graphic := nil;
            end
            else
            begin
		        	Application.MessageBox('Die Datei wurde nicht von dieser Anwendung erzeugt.', 'Datei konnte nicht geffnet werden', MB_OK);
            end;
          end;
        blrErrVersion:
        	Application.MessageBox('Es ist zwar das richtige Datenformat, aber in einer Version, die nicht untersttzt wird.', 'Datei konnte nicht geffnet werden', MB_OK);
        blrErrFormat:
        	Application.MessageBox('Die Datei enthlt kein bekanntes Datenformat.', 'Datei konnte nicht geffnet werden', MB_OK);
        blrErrCorrupt:
        	Application.MessageBox('Die Daten im Stream sind defekt.', 'Datei konnte nicht geffnet werden', MB_OK);
        else
        	Application.MessageBox('Unbekannter Rckgabeparameter.', 'Datei konnte nicht geffnet werden', MB_OK);
      end; // case
    finally
    	Data.Free;
    end;
  end;
end;


procedure TFrmMainForm.BtnSaveAsClick(Sender: TObject);
var
	Data: TRakBinaryStreamData;
  DoIt: boolean;
  FileAttr: integer;
  BmpAttr: TRakBinaryBitmap;
  str: string;
begin
	if Sdg.Execute then
  begin
  	DoIt := true;
  	if FileExists(Sdg.FileName) then
    begin // Wenn es bereits eine Datei mit dem gleichen Namen gibt
    	FileAttr := FileGetAttr(Sdg.FileName);
      if (FileAttr and SysUtils.faReadOnly) > 0 then
      begin // Wenn diese Datei schreibgeschtzt ist
        DoIt := MessageDlg('Es gibt bereits eine Datei mit diesem Namen.'
            + #13'Diese Datei ist schreibgeschtzt?'
            + #13'Soll diese berschrieben werden?', mtConfirmation,
            [mbYes, mbNo], 0) = mrYes;

        if DoIt then
        begin	// Wenn diese Datei berschrieben werden soll
        	// Dann versuchen wir den Schreibschutz aufzuheben
	      	if FileSetAttr(Sdg.FileName, FileAttr and not SysUtils.faReadOnly) <> 0 then
          begin // Wenn das nicht geklappt hat
            DoIt := false;
            Application.MessageBox('Der Schreibschutz konnte nicht aufgehoben werden.',
            	'Datei kann nicht geschrieben werden.', MB_OK);
          end;
        end;
      end
      else
      begin // Wenn diese Datei nicht schreibgeschtzt ist
        DoIt := MessageDlg('Es gibt bereits eine Datei mit diesem Namen.'
            + #13'Soll diese berschrieben werden?', mtConfirmation,
            [mbYes, mbNo], 0) = mrYes;
      end;
    end;

    if DoIt then
    begin // Wenn die Datei wirklich gespeichert werden soll
      Data := TRakBinaryStreamData.Create;
      try
        Data.ApplicationName := APP_ID;

        TRakBinaryAnsiString.AddToTag(Data, 'String-Wert', EdtString.Text);

        if trim(EdtInteger.Text) <> '' then
        	TRakBinaryLongInt.AddToTag(Data, 'Integer-Wert', StrToInt(trim(EdtInteger.Text)));
        // Wenn das Editfeld leer ist, dann Speichern wir einfach nichts
        // Wir knnten statt dessen auch einen NULL-Wert speichern.
        // Bei dem Fliekommawert machen wir das mal so

        str := InputToNormStr(EdtFloat.Text, '');
        if str = '' then
          Data.AddAttribute(TRakBinaryDouble, 'Float-Wert')
          // Da hier kein Wert bergeben wurde, wird der Wert NULL (Property IsNull = true) gesetzt.
          // Den Wert NULL darf man nicht mit dem Wert 0 verwechseln
        else
          TRakBinaryDouble.AddToTag(Data, 'Float-Wert', NormStrToFloat(str));

        TRakBinaryBoolean.AddToTag(Data, 'Boolean-Wert', CckBoolean.Checked);

        if (ImgPicture.Picture.Graphic = nil) or not (ImgPicture.Picture.Graphic is TBitmap) then
        begin
	        if ImgPicture.Picture.Graphic <> nil then
          begin
            Application.MessageBox('Das Bild wird nicht mit gespeichert,'
              + #13'da diese Version des Programms nur TBitmap untersttzt.',
              'Warnung', MB_OK);
          end
        end
        else
        begin
        	BmpAttr := TRakBinaryBitmap(Data.AddAttribute(TRakBinaryBitmap, 'Bild'));
          BmpAttr.Value := ImgPicture.Picture.Bitmap;
        end;

        try
	        Data.SaveToFile(Sdg.FileName);
        except
        	on Err: Exception do
          	begin
            	Application.MessageBox(PChar(Err.Message), 'Datei konnte nicht gespeichert werden.', MB_OK);
            end;
        end;
      finally
        Data.Free;
      end;
    end;
  end;
end;

procedure TFrmMainForm.BtnPictureClick(Sender: TObject);
begin
	if OpdPicture.Execute then
    ImgPicture.Picture.LoadFromFile(OpdPicture.FileName);
end;

procedure TFrmMainForm.CckStretchClick(Sender: TObject);
begin
	ImgPicture.Stretch := CckStretch.Checked;
end;

end.
